{***************************************************************}
{                                                               }
{  Modules creation wizards                                     }
{      mail over http http://visualdesigner.fatal.ru            }
{    http://visualdesigner.fatal.ru/                            }
{    Copyright (c) 2000-2003 Pfaffenrot S.                      }
{ ------------------------------------------------------------- }
{            home page      : http://VisulDesigner.fatal.ru/    }
{ ------------------------------------------------------------- }
{***************************************************************}
unit UWzds;

interface

//{$OPTIMIZATION OFF}

uses Classes, ToolsAPI;

type
  TSimpleCreator = class(TInterfacedObject, IOTACreator)
  protected
    procedure GetStreamSource(AStream: TStream;var Res: string);
    procedure SetChanges(var AStr: string;const AKey,AVal: string);
  public
    function GetCreatorType: string; virtual; abstract;
    function GetExisting: Boolean;
    function GetFileSystem: string;
    function GetOwner: IOTAModule; virtual; 
    function GetUnnamed: Boolean;

    property CreatorType: string read GetCreatorType;
    property Existing: Boolean read GetExisting;
    property FileSystem: string read GetFileSystem;
    property Owner: IOTAModule read GetOwner;
    property Unnamed: Boolean read GetUnnamed;
  end;

  TPGCreator = class(TSimpleCreator, IOTAProjectGroupCreator)
  public
    //constructor Create(ALoaderPath, AIniFile: PChar);
    //destructor Destroy; override;
    {IOTACreator}
    function GetOwner: IOTAModule; override;
    function GetCreatorType: string; override;
    {IOTAProjectGroupCreator}
    function GetFileName: string;
    function GetShowSource: Boolean;
    function NewProjectGroupSource(const ProjectGroupName: string): IOTAFile; deprecated;

    property FileName: string read GetFileName;
    property ShowSource: Boolean read GetShowSource;
  end;


  TPrjtCreator = class(TSimpleCreator, IOTAProjectCreator, IOTAProjectCreator50
    (*, {2006}IOTAProjectCreator80{2006}*))
  protected
    FLoaderPath, FIniFile: PChar;
    HInst: cardinal;
    PG : IOTAProjectGroup;
  public
    constructor Create(ALoaderPath, AIniFile: PChar; APG: IOTAProjectGroup);
    destructor Destroy; override;
    {IOTACreator}
    function GetOwner: IOTAModule; override;
    function GetCreatorType: string; override;

    {IOTAProjectCreator}
    function GetFileName: string;
    function GetOptionFileName: string; deprecated;
    function GetShowSource: Boolean;
    procedure NewDefaultModule; deprecated;
    function NewOptionSource(const ProjectName: string): IOTAFile; deprecated;
    procedure NewProjectResource(const Project: IOTAProject);
    function NewProjectSource(const ProjectName: string): IOTAFile;

    property FileName: string read GetFileName;
    property OptionFileName: string read GetOptionFileName;
    property ShowSource: Boolean read GetShowSource;
    {IOTAProjectCreator50}
    procedure NewDefaultProjectModule(const Project: IOTAProject);
    {IOTAProjectCreator80}
    function GetProjectPersonality: string;
    property ProjectPersonality: string read GetProjectPersonality;

  end;

  TModlCreator = class(TSimpleCreator, IOTAModuleCreator)
  protected
    function IsExist(AName: string; const AModeF: boolean): boolean;
  public
    FMode: byte; //0- Form, 1 - TDataModule
    FActiveProject: IOTAProject;
    FUnitName: PString;
    function GetCreatorType: string; override;
    function GetOwner: IOTAModule; override;

    function GetAncestorName: string;
    function GetImplFileName: string;
    function GetIntfFileName: string;
    function GetFormName: string;
    function GetMainForm: Boolean;
    function GetShowForm: Boolean;
    function GetShowSource: Boolean;
    function NewFormFile(const FormIdent, AncestorIdent: string): IOTAFile;
    function NewImplSource(const ModuleIdent, FormIdent, AncestorIdent: string): IOTAFile;
    function NewIntfSource(const ModuleIdent, FormIdent, AncestorIdent: string): IOTAFile;
    procedure FormCreated(const FormEditor: IOTAFormEditor);
    constructor Create(AKind: integer; AActiveProject: IOTAProject;  AUnitName: PString = nil);
  end;


implementation

uses SysUtils, ZLib, Forms, Windows;

const
  RT_RCDATA: PChar = 'RT_RCDATA';
const
  cFormName = 'FormName';
  cUnitName = 'UnitName';
  cAncestor = 'Ancestor';

type
  TCompRec = record
    FreezSize, FullSize, NextPoint: int64;
  end;


function GetActiveProjectGroup: IOTAProjectGroup;
var
  ModuleServices: IOTAModuleServices;
  i: Integer;
begin
  Result := nil;
  if Assigned(BorlandIDEServices) then
  begin
    ModuleServices := BorlandIDEServices as IOTAModuleServices;
    for i := 0 to ModuleServices.ModuleCount - 1 do
      if Supports(ModuleServices.Modules[i], IOTAProjectGroup, Result) then
        Break;
  end;
end;


function TSimpleCreator.GetExisting: Boolean;
begin
  Result := False;
end;

function TSimpleCreator.GetFileSystem: string;
begin
  SetLength(Result, 0);
end;

function TSimpleCreator.GetOwner: IOTAModule;
begin
  Result := nil;
end;


procedure TSimpleCreator.GetStreamSource(AStream: TStream;var Res: string);
var
  D: TDecompressionStream;
  Size: TCompRec;
begin
  SetLength(Res, 0);
  Try
    AStream.ReadBuffer(pointer(@Size)^, SizeOf(TCompRec));
    D := TDecompressionStream.Create(AStream);
    SetLength(Res, Size.FullSize+1);
    D.ReadBuffer(pointer(Res)^, Size.FullSize);
  Finally
    D.Free;
    AStream.Free;
  End;
end;

procedure TSimpleCreator.SetChanges(var AStr: string; const AKey,
  AVal: string);
var
  i : integer;
begin
  i := pos(AKey, AStr);
  if i > 0 then
  repeat
    delete(AStr, i, Length(AKey));
    insert(AVal, AStr, i);
    i := pos(AKey, AStr);
  until i < 1;
end;

function TSimpleCreator.GetUnnamed: Boolean;
begin
  Result := True;
end;

function TPrjtCreator.GetOptionFileName: string;
begin
  Result := '';//inherited GetOptionFileName;
end;

{ TPrjtCreator }

constructor TPrjtCreator.Create;
begin
  inherited Create;
  FLoaderPath := ALoaderPath;
  FIniFile := AIniFile;
  PG := APG;
//  MessageBox(0, 'p1', nil, 0);
end;

destructor TPrjtCreator.Destroy;
begin
  inherited;
end;

function TPrjtCreator.GetCreatorType: string;
begin
  Result := '';//sLibrary;
end;

function TPrjtCreator.GetFileName: string;
begin
  Result := '';
end;

function TPrjtCreator.GetOwner: IOTAModule;
begin
  Result := nil;//PG;
end;

function TPrjtCreator.GetProjectPersonality: string;
begin
end;

function TPrjtCreator.GetShowSource: Boolean;
begin
  Result := True;
end;

procedure TPrjtCreator.NewDefaultModule;
//var
//  Module: IOTAModule;
//  ModuleCreator: TModlCreator;
begin
  inherited;
//  ModuleCreator := TModlCreator.Create(2, nil);
//  Module := (BorlandIDEServices as IOTAModuleServices).CreateModule(ModuleCreator);
end;

procedure TPrjtCreator.NewDefaultProjectModule(const Project: IOTAProject);
var
  Module: IOTAModule;
  ModuleCreator: TModlCreator;
  UnitName: string;
begin
  ModuleCreator := TModlCreator.Create(2, Project, @UnitName);
  Module := (BorlandIDEServices as IOTAModuleServices).CreateModule(ModuleCreator);
  //Project.Rename(UnitName+'', 'GeneralLeakPatch.pas');
//  Project.AddFile('GeneralLeakPatch', True);
end;

function TPrjtCreator.NewOptionSource(const ProjectName: string): IOTAFile;
begin
  Result := nil; //     
end;

procedure TPrjtCreator.NewProjectResource(const Project: IOTAProject);
begin
////  Project.ProjectOptions.EditOptions;
////  Project.ProjectOptions.Values[] := ;
//  Project.ProjectOptions.Values['OutputDir'] := ExtractFilePath(string(FIniFile));
//  Project.ProjectOptions.Values['HostApplication'] := string(FLoaderPath);
//  Project.ProjectOptions.Values['RunParams'] := string(FIniFile);
////  Project.ProjectOptions.Values[] := ;
end;

{$R _TempRes.RES}


function TPrjtCreator.NewProjectSource(const ProjectName: string): IOTAFile;
var
  S : string;
  R : TStream;
begin
  //       
  R := TResourceStream.Create(HInstance, '_TempDpr', RT_RCDATA);
  GetStreamSource(R, S);
  SetChanges(S, '%' + cUnitName +'%', ProjectName);
  Result := TOTAFile.Create(S);
end;

{ TModlCreator }

constructor TModlCreator.Create;
begin
  inherited Create;
  FUnitName :=  AUnitName;
  FMode := byte(AKind);
  FActiveProject := AActiveProject;
end;

procedure TModlCreator.FormCreated(const FormEditor: IOTAFormEditor);
begin
end;

function TModlCreator.GetAncestorName: string;
begin
  SetLength(Result, 0);
  case FMode of
    0: Result := 'TForm';
    1: Result := 'TDataModule';
  end;
end;

function TModlCreator.GetCreatorType: string;
begin
  Result := sForm;//sUnit;
end;

function TModlCreator.GetFormName: string;
begin
  Result := '';
end;

function TModlCreator.GetImplFileName: string;
begin
  SetLength(Result, 0);
//  if FMode = 2 then
//    Result := 'GeneralLeakPatch.pas';
end;

function TModlCreator.GetIntfFileName: string;
begin
  SetLength(Result, 0);
end;

function TModlCreator.GetMainForm: Boolean;
begin
  Result := False;
end;

function TModlCreator.GetOwner: IOTAModule;
begin
  Result := GetActiveProject;
end;

function TModlCreator.GetShowForm: Boolean;
begin
  Result := True;
end;

function TModlCreator.GetShowSource: Boolean;
begin
  Result := True;
end;

function TModlCreator.IsExist(AName: string; const AModeF: boolean): boolean;
 function IsExistinProj(AP : IOTAProject; AModeF: Boolean): boolean;
 var i : integer;
 begin
   Result := False;
   if AP <> nil then
   for i := 0 to AP.GetModuleCount - 1 do
     case AModeF of
      False: if AP.GetModule(i).FileName = AName then
           Result := True;
      True: if AP.GetModule(i).FormName = AName then
           Result := True;
     end;
 end;
var
 PG : IOTAProjectGroup;
 P : IOTAProject;
 i : integer;
begin
  Result := False;
  PG := GetActiveProjectGroup;
  //(BorlandIDEServices as IOTAModuleServices).MainProjectGroup;
  if PG = nil then
  begin
    P := FActiveProject;//GetActiveProject;
    Result := IsExistinProj(P, AModeF);
  end
  else
    for i := 0 to PG.ProjectCount - 1 do
      if IsExistinProj(PG.Projects[i], AModeF) then
        Result := True;
end;


function TModlCreator.NewFormFile(const FormIdent,
  AncestorIdent: string): IOTAFile;
var
  S : string;
  R : TStream;
begin
  //       
  //MessageBox(0, PChar(FormIdent + AncestorIdent), nil, 0);
  case FMode of
    0: R := TResourceStream.Create(HInstance, '_TempFrmDfm', RT_RCDATA);
    1: R := TResourceStream.Create(HInstance, '_TempDMDfm', RT_RCDATA);
    2: R := nil;
  end;
  Result := nil;
  if R <> nil then
  begin
    GetStreamSource(R, S);
    SetChanges(S, '%' + cAncestor +'%', FormIdent);
    SetChanges(S, '%' + cFormName +'%', copy(FormIdent, 2, length(FormIdent)));
    Result := TOTAFile.Create(S);
  end;
end;

function TModlCreator.NewImplSource(const ModuleIdent, FormIdent,
  AncestorIdent: string): IOTAFile;
var
  S : string;
  R : TStream;
begin
  //       
  case FMode of
    0: R := TResourceStream.Create(HInstance, '_TempFrmPas', RT_RCDATA);
    1: R := TResourceStream.Create(HInstance, '_TempDMPas', RT_RCDATA);
    2: R := TResourceStream.Create(HInstance, '_GLP', RT_RCDATA);
  end;
  if FUnitName <> nil then
    FUnitName^ := ModuleIdent;
  GetStreamSource(R, S);
  SetChanges(S, '%' + cUnitName +'%', ModuleIdent);
  SetChanges(S, '%' + cAncestor +'%', AncestorIdent);
  SetChanges(S, '%' + cFormName +'%', FormIdent);
  Result := TOTAFile.Create(S);
end;

function TModlCreator.NewIntfSource(const ModuleIdent, FormIdent,
  AncestorIdent: string): IOTAFile;
begin
  Result := nil;
end;

{ TPGCreater }

{---------------------------------------------}

procedure CreatePrj(ABorland, FLoaderPath, FIniFileName: pointer); stdcall;
var
  ModuleServices: IOTAModuleServices;
  PrjCreator: TPrjtCreator;
  APG : IOTAProjectGroup;
  i : integer;
  Module: IOTAModule;
begin
  ModuleServices := nil;
  if pointer(BorlandIDEServices) <> ABorland then
    BorlandIDEServices := IBorlandIDEServices(ABorland);
  try
    if Assigned(BorlandIDEServices) and
     Supports(BorlandIDEServices, IOTAModuleServices, ModuleServices) then
    begin

      {$IFDEF VER180}
      try
        if ModuleServices <> nil then
          APG := ModuleServices.MainProjectGroup;
      except
        APG := nil;
      end;
      {$ELSE}
      if ModuleServices <> nil then
      for i := 0 to ModuleServices.ModuleCount - 1 do
      begin
        Module := ModuleServices.Modules[I];
        if Supports(Module, IOTAProjectGroup, APG) then
          Break;
      end;
      {$ENDIF}
      if APG = nil then
      begin
        APG := ModuleServices.CreateModule(TPGCreator.Create) as IOTAProjectGroup;
     end;
     //if APG = nil then
        //MessageBox(0, '2', nil, 0);



      PrjCreator := TPrjtCreator.Create(PChar(FLoaderPath), PChar(FIniFileName), APG);

      ModuleServices.CreateModule(PrjCreator);
        //MessageBox(0, '4', nil, 0);
    end;
  finally
    ModuleServices := nil;
  end;
end;
procedure CreateMod(ABorland, AKind, AProject: pointer); stdcall;
var
  ModuleServices: IOTAModuleServices;
  ModuleCreater: TModlCreator;
begin
  ModuleServices := nil;
  if pointer(BorlandIDEServices) <> ABorland then
    BorlandIDEServices := IBorlandIDEServices(ABorland);
  try
    if Assigned(BorlandIDEServices) and
     Supports(BorlandIDEServices, IOTAModuleServices, ModuleServices) then
    begin
      ModuleCreater := TModlCreator.Create(integer(AKind),IOTAProject(AProject));
      ModuleServices.CreateModule(ModuleCreater);
    end;
  finally
    ModuleServices := nil;
  end;
end;
exports CreatePrj name 'CP', CreateMod name 'CM';

{$OPTIMIZATION ON}

{ TPGCreator }

function TPGCreator.GetCreatorType: string;
begin
  Result := 'ProjectGroup';
end;

function TPGCreator.GetFileName: string;
begin
  Result := '';
end;

function TPGCreator.GetOwner: IOTAModule;
begin
  Result := nil;//(BorlandIDEServices as IOTAModuleServices);
end;

function TPGCreator.GetShowSource: Boolean;
begin
  Result := False;
end;

function TPGCreator.NewProjectGroupSource(
  const ProjectGroupName: string): IOTAFile;
begin
  Result := nil;//TOTAFile.Create('PROJECT');
end;

end.
